@@ -11,21 +11,25 @@ core functionality for deserializing MMIF JSON data into live objects
 and serializing live objects into MMIF JSON data. Specialized behavior
 for the different components of MMIF is added in the subclasses.
 """
-
 import json
 from datetime import datetime
+from types import MethodType
 from typing import Union, Any, Dict, Optional, TypeVar, Generic, Generator, Iterator, Type, Set
 
 from deepdiff import DeepDiff
 
+import mmif
+
 T = TypeVar('T')
 S = TypeVar('S')
+JSON_PRMTV_TYPES: Type = Union[str, int, float, bool, None]
 
 __all__ = [
     'MmifObject',
     'MmifObjectEncoder',
     'DataList',
-    'DataDict'
+    'DataDict',
+    'JSON_PRMTV_TYPES'
 ]
 
 
@@ -77,6 +81,8 @@ class MmifObject(object):
      an ID value automatically generated, based on its parent object.
     """
     
+    # these are the reserved names that cannot be used as attribute names, and 
+    # they won't be serialized
     reserved_names: Set[str] = {
         'reserved_names',
         '_unnamed_attributes',
@@ -99,6 +105,8 @@ class MmifObject(object):
             self._unnamed_attributes = {}
         if mmif_obj is not None:
             self.deserialize(mmif_obj)
+        for method in mmif.patches[self.__class__]:
+            setattr(self, method.__name__, MethodType(method, self))
 
     def disallow_additional_properties(self) -> None:
         """
@@ -419,7 +427,7 @@ class DataList(MmifObject, Generic[T]):
         return self._items.__len__()
 
     def __reversed__(self) -> Iterator[T]:
-        return reversed(list(self._items.values()))
+        return reversed(self._items.values())
 
     def __contains__(self, item) -> bool:
         return item in self._items
@@ -440,9 +448,6 @@ class DataDict(MmifObject, Generic[T, S]):
     def _serialize(self, *args, **kwargs) -> dict:
         return super()._serialize(self._items)
 
-    def _deserialize(self, input_dict: dict) -> None:
-        raise NotImplementedError()
-
     def get(self, key: T, default=None) -> Optional[S]:
         return self._items.get(key, default)
 
